/*
 * Beangle, Agile Java/Scala Development Scaffold and Toolkit
 *
 * Copyright (c) 2005-2013, Beangle Software.
 *
 * Beangle is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Beangle is distributed in the hope that it will be useful.
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Beangle.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.beangle.commons.dao.query.builder;

import java.util.*;

import org.beangle.commons.bean.PropertyUtils;
import org.beangle.commons.collection.CollectUtils;
import org.beangle.commons.entity.Component;
import org.beangle.commons.entity.Entity;
import org.beangle.commons.entity.util.ValidEntityKeyPredicate;
import org.beangle.commons.lang.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 条件提取辅助类
 * 
 * @author chaostone
 */
public final class ConditionUtils {
  private static final Logger logger = LoggerFactory.getLogger(ConditionUtils.class);

  private ConditionUtils() {
    super();
  }

  public static String toQueryString(final List<Condition> conditions) {
    if (null == conditions || conditions.isEmpty()) { return ""; }
    final StringBuilder buf = new StringBuilder("");
    for (final Iterator<Condition> iter = conditions.iterator(); iter.hasNext();) {
      final Condition con = iter.next();
      buf.append('(').append(con.getContent()).append(')');
      if (iter.hasNext()) {
        buf.append(" and ");
      }
    }
    return buf.toString();
  }

  /**
   * 提取对象中的条件<br>
   * 提取的属性仅限"平面"属性(允许包括component)<br>
   * 过滤掉属性:null,或者空Collection
   * 
   * @param alias
   * @param entity
   */
  public static List<Condition> extractConditions(final String alias, final Entity<?> entity) {
    if (null == entity) { return Collections.emptyList(); }
    final List<Condition> conditions = new ArrayList<Condition>();

    StringBuilder aliasBuilder = new StringBuilder(alias == null ? "" : alias);
    if (aliasBuilder.length() > 0 && !alias.endsWith(".")) aliasBuilder.append(".");
    String attr = "";
    try {
      final Set<String> props = PropertyUtils.getWritableProperties(entity.getClass());
      for (final Iterator<String> iter = props.iterator(); iter.hasNext();) {
        attr = iter.next();
        final Object value = PropertyUtils.getProperty(entity, attr);
        if (null == value) continue;
        if (!(value instanceof Collection<?>)) addAttrCondition(conditions, alias + attr, value);
      }
    } catch (Exception e) {
      logger.debug("error occur in extractConditions for  bean {} with attr named {}", entity, attr);
    }
    return conditions;
  }

  /**
   * 获得条件的绑定参数映射
   * 
   * @param conditions
   */
  public static Map<String, Object> getParamMap(final List<Condition> conditions) {
    final Map<String, Object> params = new HashMap<String, Object>();
    for (final Condition con : conditions) {
      params.putAll(getParamMap(con));
    }
    return params;
  }

  /**
   * 获得条件的绑定参数映射
   * 
   * @param condition
   */
  public static Map<String, Object> getParamMap(final Condition condition) {
    final Map<String, Object> params = new HashMap<String, Object>();
    if (!Strings.contains(condition.getContent(), "?")) {
      final List<String> paramNames = condition.getParamNames();
      if (paramNames.size() > condition.getParams().size()) { throw new RuntimeException(
          "condition params not set [" + condition.getContent() + "] with value:" + condition.getParams()); }
      for (int i = 0; i < paramNames.size(); i++) {
        params.put(paramNames.get(i), condition.getParams().get(i));
      }
    }
    return params;
  }

  /**
   * 为extractConditions使用的私有方法<br>
   * 
   * @param conditions
   * @param name
   * @param value
   * @param mode
   */
  private static void addAttrCondition(final List<Condition> conditions, final String name, Object value) {
    if (value instanceof String) {
      if (Strings.isBlank((String) value)) { return; }
      StringBuilder content = new StringBuilder(name);
      content.append(" like :").append(name.replace('.', '_'));
      conditions.add(new Condition(content.toString(), "%" + value + "%"));
    } else if (value instanceof Component) {
      conditions.addAll(extractComponent(name, (Component) value));
      return;
    } else if (value instanceof Entity<?>) {
      try {
        final String key = "id";
        Object property = PropertyUtils.getProperty(value, key);
        if (ValidEntityKeyPredicate.Instance.apply(property)) {
          StringBuilder content = new StringBuilder(name);
          content.append('.').append(key).append(" = :").append(name.replace('.', '_')).append('_')
              .append(key);
          conditions.add(new Condition(content.toString(), property));
        }
      } catch (Exception e) {
        logger.warn("getProperty " + value + "error", e);
      }
    } else {
      conditions.add(new Condition(name + " = :" + name.replace('.', '_'), value));
    }
  }

  private static List<Condition> extractComponent(final String prefix, final Component component) {
    if (null == component) { return Collections.emptyList(); }
    final List<Condition> conditions = CollectUtils.newArrayList();
    String attr = "";
    try {
      final Set<String> props = PropertyUtils.getWritableProperties(component.getClass());
      for (final Iterator<String> iter = props.iterator(); iter.hasNext();) {
        attr = iter.next();
        final Object value = PropertyUtils.getProperty(component, attr);
        if (value == null) {
          continue;
        } else if (value instanceof Collection<?>) {
          if (((Collection<?>) value).isEmpty()) continue;
        } else {
          addAttrCondition(conditions, prefix + "." + attr, value);
        }
      }
    } catch (Exception e) {
      logger.warn("error occur in extractComponent of component:" + component + "with attr named :" + attr);
    }
    return conditions;
  }

}
